void
#if defined (RENDER_SPR_MMC5)
#  ifdef RENDER_SPR_SKIP
NES_PPU::render_spr_mmc5_skip()
#  else
NES_PPU::render_spr_mmc5 (uint16 *buf)
#  endif
#elif defined (RENDER_SPR_MMC2)
#  ifdef RENDER_SPR_SKIP
NES_PPU::render_spr_mmc2_skip()
#  else
NES_PPU::render_spr_mmc2 (uint16 *buf)
#  endif
#else
NES_PPU::render_spr_regular (uint16 *buf)
#endif
{
  int sprite_index;
  uint32 spr_y;
  uint8* spr;
  
  int spr_x, start_x, end_x;
  int num_sprites;
  
  uint32 spr_height = sprite_height ();
  
  /* rendered tile pattern */
  uint8 tile_pattern[8]; 
  
  /* pre-calced sprite data */
  sprites_per_line *spl = spr_per_line + current_frame_line;
  uint8 *spr_index = spl->sprite_index;
  
#ifdef RENDER_SPR_MMC5
  parent_NES->mapper->PPU_Latch_RenderScreen(0,0);
#endif
  
  num_sprites = spl->num;
  while (num_sprites--)
  {
    sprite_index = *spr_index++;
    spr = spr_ram + (sprite_index * 4);
    
#ifndef RENDER_SPR_SKIP    
    spr_y = spr[0] + 1;
    
    /* get x coord */
    spr_x = spr[3];
    
    start_x = 0;
    end_x = 8;
    
    /* clip left */
    if((spr_x < 8) && (spr_clip_left8()))
    {
      if (0 == spr_x) continue;
      start_x += (8 - spr_x);
    }
    
    /* clip right */
    if ((spr_x + 7) > 255)
      end_x -= ((spr_x + 7) - 255);
#endif
    
#ifdef RENDER_SPR_MMC2    
    {
      uint32 addr = spr[1] << 4;
      
      /* if ((addr & 0x0FF0) == 0x0FD0 || (addr & 0x0FF0) == 0x0FE0) */
      if (((((addr >> 4) & 0xff) - 0xfd) & 0xfe) == 0)
	parent_NES->mapper->PPU_Latch_FDFE(addr);
    }
#endif
    
#ifndef RENDER_SPR_SKIP
    {
      /* make tile_pattern */ 
      uint32 tile_addr;
      uint8 p1, p2;
      uint32 y;
      uint8 *t;
      
      y = current_frame_line - spr_y;
      
      /* vertically flip */
      if (spr[2] & 0x80)
	y = (spr_height - 1) - y;
      
      if (spr_height == 8)
	tile_addr = (spr[1] << 4) + spr_pattern_table_addr + (y & 0x07);
      else
      {
	tile_addr = (spr[1] << 4) + ((spr[1] & 0x01) << 12) + (y & 0x07);
	if (y < 8)
	  tile_addr -= (spr[1] & 0x01) << 4;
	else
	  tile_addr += (!(spr[1] & 0x01)) << 4;
      }
      
      p1 = (VRAM(tile_addr + 8) & 0xaa) | ((VRAM(tile_addr) >> 1) & 0x55);
      p2 = ((VRAM(tile_addr + 8) << 1) & 0xaa) | (VRAM(tile_addr) & 0x55);
      
      /* horizontally filp */
      if (spr[2] & 0x40)
      {
	t = tile_pattern;
	
	*t++ = p2 & 0x03; /* 0 */
	*t++ = p1 & 0x03; /* 1 */
	p1 >>= 2; p2 >>= 2;
	
	*t++ = p2 & 0x03; /* 2 */
	*t++ = p1 & 0x03; /* 3 */
	p1 >>= 2; p2 >>= 2;
	
	*t++ = p2 & 0x03; /* 4 */
	*t++ = p1 & 0x03; /* 5 */
	p1 >>= 2; p2 >>= 2;
	
	*t++ = p2 & 0x03; /* 6 */
	*t++ = p1 & 0x03; /* 7 */
      }
      else
      {
	t = tile_pattern + 8;
	
	*(--t) = p2 & 0x03; /* 7 */
	*(--t) = p1 & 0x03; /* 6 */
	p2 >>= 2; p1 >>= 2;
	
	*(--t) = p2 & 0x03; /* 5 */
	*(--t) = p1 & 0x03; /* 4 */
	p2 >>= 2; p1 >>= 2;
	
	*(--t) = p2 & 0x03; /* 3 */
	*(--t) = p1 & 0x03; /* 2 */
	p2 >>= 2; p1 >>= 2;
	
	*(--t) = p2 & 0x03; /* 1 */
	*(--t) = p1 & 0x03; /* 0 */
      }
    }
    
    {
      uint16 *tile_pallete = spr_pal_color + ((spr[2] & 0x03) << 2);
      uint16 *p = buf + SIDE_MARGIN + spr_x + start_x;
      uint8 *s = solid_buf + SIDE_MARGIN + spr_x + start_x;
      uint8 *t = tile_pattern + start_x;

      /* set sprite #0 hit flag */
      if (!sprite_index)
      {
	int x;
	uint8 *s_tmp = s;
	uint8 *t_tmp = t;
	
	x = end_x - start_x;
	while (x)
	{
	  if (*t_tmp && (*s_tmp & BG_WRITTEN_FLAG))
	  {
	    LowRegs[2] |= 0x40;
	    break;
	  }
	  
	  ++s_tmp;
	  ++t_tmp;
	  --x;
	}
      }
      
      /* render sprite */
      if (spr[2] & 0x20)
      {
	int x;
	x = end_x - start_x;
	while (x)
	{
	  if (*t && !(*s & SPR_WRITTEN_FLAG))
	  {
	    *s |= SPR_WRITTEN_FLAG;
	    if (!(*s & BG_WRITTEN_FLAG))
	      *p = *(tile_pallete + *t);
	  }
	  
	  ++p; 
	  ++s;
	  ++t;
	  --x;
	}
      }
      else
      {
	int x;
	x = end_x - start_x;
	while (x)
	{
	  if (*t && !(*s & SPR_WRITTEN_FLAG))
	  {
	    *p = *(tile_pallete + *t);
	    *s |= SPR_WRITTEN_FLAG;
	  }
	  
	  ++p;
	  ++s;
	  ++t;
	  --x;
	}
      }
    }
#endif
  }
  
  if (spl->num > 8)
    LowRegs[2] |= 0x20;
  else
    LowRegs[2] &= 0xDF;
}


